
package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.service.ReexamineLogService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.reexaminelog.ReexamineLogInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.reexaminelog.ReexamineLogQuery;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

@RestController
@Api(tags = "重考设置记录相关接口")
@RequestMapping("${$rmp.ctr.oe}/reexamine/log")
public class ReexamineLogController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ReexamineLogController.class);

    @Autowired
    private ReexamineLogService reexamineLogService;

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @PostMapping("/list")
    @ApiOperation(value = "查询“重考设置记录”列表(分页)")
    public Page<ReexamineLogInfo> list(ReexamineLogQuery query) {
        UserDataRule courseRule = super.getUserDataRule(DataRuleType.COURSE);
        UserDataRule orgRule = super.getUserDataRule(DataRuleType.ORG);
        return reexamineLogService.getReexamineLogList(query, courseRule, orgRule);
    }

    @DataRule(type = {DataRuleType.COURSE, DataRuleType.ORG})
    @GetMapping("/list/export")
    @ApiOperation(value = "导出“重考设置记录”列表(Excel)")
    public void export(ReexamineLogQuery query, HttpServletResponse response) throws Exception {
        UserDataRule courseRule = super.getUserDataRule(DataRuleType.COURSE);
        UserDataRule orgRule = super.getUserDataRule(DataRuleType.ORG);
        query.setByExport(true);

        Page<ReexamineLogInfo> page = reexamineLogService.getReexamineLogList(query, courseRule, orgRule);
        List<ReexamineLogInfo> list = new ArrayList<>(page.getContent());
        ExportUtils.exportEXCEL("重考设置记录", ReexamineLogInfo.class, list, response);
    }

}
