package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import cn.com.qmth.examcloud.core.oe.admin.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.DateUtils;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordFileAnswerRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordFileAnswerEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.IsSuccess;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.TrueFalse;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDetailService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExportTaskService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordFileAnswerInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.*;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.fss.FssHelper;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ExamRecordDetailServiceImpl implements ExamRecordDetailService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordDetailServiceImpl.class);

    @Autowired
    private ExamRecordFileAnswerRepo examRecordFileAnswerRepo;

    @Autowired
    private ExportTaskService exportTaskService;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<ExamRecordInfo> getExamRecordDetailList(ExamRecordQuery query, UserDataRule courseRule, UserDataRule orgRule) {
        Check.isNull(query, "请求参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试！");

        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());
        if (courseRule.assertEmptyQueryResult() || orgRule.assertEmptyQueryResult()) {
            return Page.empty(pageable);
        }

        int offset = (query.getPageNo() - 1) * query.getPageSize();
        String countSql = this.queryExamRecordDetailListSql(query, true, courseRule, orgRule);
        String querySql = this.queryExamRecordDetailListSql(query, false, courseRule, orgRule);
        String pageSql = querySql + " limit " + offset + "," + query.getPageSize();
        log.debug(pageSql);

        Long totalElements = jdbcTemplate.queryForObject(countSql, Long.class);
        if (totalElements == null || totalElements == 0) {
            return Page.empty(pageable);
        }

        List<ExamRecordInfo> list = jdbcTemplate.query(pageSql, new BeanPropertyRowMapper(ExamRecordInfo.class));
        if (CollectionUtils.isEmpty(list)) {
            return Page.empty(pageable);
        }

        // 填充其它信息
        this.fillOtherInfo(list, query.getExamId());

        return new PageImpl<>(list, pageable, totalElements);
    }

    @Override
    public List<ExamRecordInfo> getExamRecordDetailListForExport(ExamRecordQuery query, UserDataRule courseRule, UserDataRule orgRule) {
        Check.isNull(query, "请求参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试！");

        if (courseRule.assertEmptyQueryResult() || orgRule.assertEmptyQueryResult()) {
            return new ArrayList<>();
        }

        query.setPageNo(1);
        query.setPageSize(1000);

        List<ExamRecordInfo> result = new ArrayList<>();
        while (true) {
            // 检查是否终止导出
            exportTaskService.checkStopExportTaskById(query.getTaskId());

            int offset = (query.getPageNo() - 1) * query.getPageSize();
            String querySql = this.queryExamRecordDetailListSql(query, false, courseRule, orgRule);
            String pageSql = querySql + " limit " + offset + "," + query.getPageSize();
            log.warn("导出考试明细：{}", pageSql);

            List<ExamRecordInfo> list = jdbcTemplate.query(pageSql, new BeanPropertyRowMapper(ExamRecordInfo.class));
            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            result.addAll(list);
            query.setPageNo(query.getPageNo() + 1);
        }

        // 填充其它信息
        this.fillOtherInfo(result, query.getExamId());

        return result;
    }

    private void fillOtherInfo(List<ExamRecordInfo> list, Long examId) {
        ExamSettingsCacheBean examCache = CacheHelper.getExamSettings(examId);
        Map<Long, String> orgMaps = new HashMap<>();
        Map<Long, CourseCacheBean> courseMaps = new HashMap<>();
        Map<Long, ExamStageCacheBean> examStageMaps = new HashMap<>();
        Map<Long, StudentCacheBean> studentMaps = new HashMap<>();
        boolean hidden = IdentityNumberHelper.identityNumbeConceal(examCache.getRootOrgId());

        //考试 是否开启身份检测
        boolean faceEnable = false;
        if (ExamType.ONLINE.name().equals(examCache.getExamType())) {
            ExamPropertyCacheBean property = CacheHelper.getExamProperty(examId, ExamProperties.IS_FACE_ENABLE.name());
            faceEnable = property != null && StringUtil.isTrue(property.getValue());
        }

        for (ExamRecordInfo info : list) {
            // 考试名称
            info.setExamName(examCache.getName());

            // 学习中心名称
            String orgName = orgMaps.get(info.getOrgId());
            if (orgName == null) {
                orgName = CacheHelper.getOrg(info.getOrgId()).getName();
                orgMaps.put(info.getOrgId(), orgName);
            }
            info.setOrgName(orgName);

            // 课程信息
            CourseCacheBean course = courseMaps.get(info.getCourseId());
            if (course == null) {
                course = CacheHelper.getCourse(info.getCourseId());
                courseMaps.put(info.getCourseId(), course);
            }
            info.setCourseCode(course.getCode());
            info.setCourseName(course.getName());
            info.setCourseLevel(CourseLevel.getCourseLevel(course.getLevel()).getTitle());
            info.setCourseNameAndCode(info.getCourseName() + "(" + info.getCourseCode() + ")");

            // 场次信息
            if (info.getExamStageId() != null) {
                ExamStageCacheBean examStage = examStageMaps.get(info.getExamStageId());
                if (examStage == null) {
                    examStage = CacheHelper.getExamStage(examId, info.getExamStageId());
                    examStageMaps.put(info.getExamStageId(), examStage);
                }
                info.setExamStageOrder(examStage.getStageOrder());
                info.setStageStartTime(examStage.getStartTime());
                info.setStageEndTime(examStage.getEndTime());
                info.setExamStage(examStage.getStageOrder() + "("
                        + DateUtil.format(examStage.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT) + "至"
                        + DateUtil.format(examStage.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT) + ")");
            }

            // 学生信息
            StudentCacheBean student = studentMaps.get(info.getStudentId());
            if (student == null) {
                student = CacheHelper.getStudent(info.getStudentId());
                studentMaps.put(info.getStudentId(), student);
            }
            info.setPhone(student.getPhoneNumber());

            info.setDataId(info.getId());
            info.setHasVirtual(StringUtils.isNotBlank(info.getVirtualCameraNames()));
            info.setShowReAudit(faceEnable);
            info.setStudentNumber(info.getStudentId() + "" + info.getExamStudentId());

            if (info.getSubjectiveTotalScore() == null) {
                info.setSubjectiveTotalScore("0");
            }

            //违纪标志
            info.setDisciplineSign(TrueFalse.of(info.getIsWarn()));
            info.setIsIllegality("1".equals(info.getIsIllegality()) ? "是" : "否");

            if (info.getStartTime() != null) {
                info.setPaperStartTime(DateUtils.format(info.getStartTime()));
            } else {
                info.setPaperStartTime("");
            }
            if (info.getEndTime() != null) {
                info.setPaperSubmitTime(DateUtils.format(info.getEndTime()));
                info.setIsSubmit("是");
            } else {
                info.setPaperSubmitTime("");
                info.setIsSubmit("否");
            }

            //每次考试持续时间
            if (info.getStartTime() != null && info.getEndTime() != null) {
                if (info.getUsedExamTime() != null) {
                    info.setExamTime(info.getUsedExamTime() > 0 ? DateUtils.diff(info.getUsedExamTime()) : "");
                }
            } else {
                info.setExamTime("");
            }

            IsSuccess result = IsSuccess.strToEnum(info.getFaceVerifyResult());
            if (result != null) {
                info.setFaceVerifyResult(result.getDesc());
            }

            if (ExamType.OFFLINE.name().equals(examCache.getExamType())) {
                // 离线考试作答文件
                List<ExamRecordFileAnswerEntity> fileAnswers = examRecordFileAnswerRepo.findByExamRecordDataId(info.getId());
                info.setOfflineFiles(this.ofFileAnswer(fileAnswers));
            }

            if (hidden) {
                info.setIdentityNumber(IdentityNumberHelper.conceal(info.getIdentityNumber()));
            }
        }
    }

    private List<ExamRecordFileAnswerInfo> ofFileAnswer(List<ExamRecordFileAnswerEntity> entities) {
        List<ExamRecordFileAnswerInfo> result = new ArrayList<>();

        for (ExamRecordFileAnswerEntity entity : entities) {
            ExamRecordFileAnswerInfo info = new ExamRecordFileAnswerInfo();
            info.setId(entity.getId());
            info.setExamRecordDataId(entity.getExamRecordDataId());
            info.setOfflineFileName(entity.getFileName());
            info.setOriginalFileName(entity.getOriginalFileName());
            info.setFileType(entity.getFileType());
            info.setSuffix(entity.getSuffix());
            info.setProperties(entity.getProperties());

            info.setOfflineFileUrl(FssHelper.finalFileUrl(entity.getFileUrl()));
            result.add(info);
        }

        return result;
    }

    private String queryExamRecordDetailListSql(ExamRecordQuery query, boolean isCount
            , UserDataRule courseRule, UserDataRule orgRule) {
        StringBuilder sql = new StringBuilder();
        if (isCount) {
            sql.append(" select count(DISTINCT rd.id)");
        } else {
            sql.append(" select rd.*,a.audit_user_name,sco.objective_score objectiveTotalScore,");
            sql.append(" sco.subjective_score subjectiveTotalScore,sco.total_score paperTotalScore,");
            sql.append(" es.grade,es.specialty_name,");
            sql.append(" (select GROUP_CONCAT(DISTINCT pr.source_ip) from ec_oe_exam_process_record pr")
                    .append(" where pr.exam_record_data_id = rd.id) ip,");
            sql.append(" (select GROUP_CONCAT(DISTINCT cm.name) from ec_oe_exam_capture_camera_info cm")
                    .append(" where cm.exam_record_data_id = rd.id and cm.virtual_camera = 1) virtualCameraNames");
        }

        sql.append(" from ec_oe_exam_record_data rd");
        sql.append(" left join ec_oe_exam_student es on es.exam_student_id = rd.exam_student_id");
        sql.append(" left join ec_oe_exam_audit a on a.exam_record_data_id = rd.id");
        sql.append(" left join ec_oe_exam_score sco on sco.exam_record_data_id = rd.id");

        sql.append(" where rd.exam_id = ").append(query.getExamId());
        sql.append(" and rd.exam_record_status in ('EXAM_END','EXAM_OVERDUE')");

        if (query.getExamRecordId() != null) {
            sql.append(" and rd.id = ").append(query.getExamRecordId());
        }

        sql.append(" and ((rd.is_warn = 0) or (rd.is_warn = 1 and rd.is_audit = 1))");

        if (query.getIsIllegality() != null) {
            sql.append(" and rd.is_illegality = ").append(query.getIsIllegality() ? "1" : "0");
        }

        if (courseRule.assertNeedQueryRefIds()) {
            // 限定课程数据权限范围
            if (query.getCourseId() != null) {
                if (courseRule.getRefIds().contains(query.getCourseId())) {
                    sql.append(" and rd.course_id = ").append(query.getCourseId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.course_id = -1");
                }
            } else {
                sql.append(" and rd.course_id in (").append(StringUtils.join(courseRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getCourseId() != null) {
                sql.append(" and rd.course_id = ").append(query.getCourseId());
            }
        }

        if (orgRule.assertNeedQueryRefIds()) {
            // 限定机构数据权限范围
            if (query.getOrgId() != null) {
                if (orgRule.getRefIds().contains(query.getOrgId())) {
                    sql.append(" and rd.org_id = ").append(query.getOrgId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.org_id = -1");
                }
            } else {
                sql.append(" and rd.org_id in (").append(StringUtils.join(orgRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getOrgId() != null) {
                sql.append(" and rd.org_id = ").append(query.getOrgId());
            }
        }

        if (query.getExamStageId() != null) {
            sql.append(" and rd.exam_stage_id = ").append(query.getExamStageId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and rd.course_level = '").append(query.getCourseLevel()).append("'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and rd.identity_number like '").append(query.getIdentityNumber()).append("%'");
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and rd.student_code like '").append(query.getStudentCode()).append("%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and rd.student_name like '").append(query.getStudentName()).append("%'");
        }
        if (StringUtils.isNotBlank(query.getInfoCollector())) {
            sql.append(" and rd.info_collector like '").append(query.getInfoCollector()).append("%'");
        }

        if (StringUtils.isNotBlank(query.getStartTime()) && StringUtils.isNotBlank(query.getEndTime())) {
            sql.append(" and rd.start_time >= '").append(query.getStartTime()).append("'");
            sql.append(" and rd.start_time <= '").append(query.getEndTime()).append("'");
        }

        if (query.getSwitchScreenCountStart() != null) {
            sql.append(" and rd.switch_screen_count >= ").append(query.getSwitchScreenCountStart());
        }
        if (query.getSwitchScreenCountEnd() != null) {
            sql.append(" and rd.switch_screen_count <= ").append(query.getSwitchScreenCountEnd());
        }

        if (StringUtils.isNotBlank(query.getAuditUserName())) {
            sql.append(" and a.audit_user_name like '").append(query.getAuditUserName()).append("%'");
        }

        if (StringUtils.isNotBlank(query.getIp())) {
            sql.append(" and exists (");
            sql.append(" select 1 from ec_oe_exam_process_record pr where pr.exam_record_data_id = rd.id");
            sql.append(" and pr.source_ip = '").append(query.getIp()).append("'");
            sql.append(" )");
        }

        if (query.getHasVirtual() != null) {
            sql.append(" and").append(query.getHasVirtual() ? " exists" : " not exists");
            sql.append(" (");
            sql.append(" select 1 from ec_oe_exam_capture_camera_info v where v.exam_record_data_id = rd.id");
            sql.append(" and v.virtual_camera = 1");
            if (StringUtils.isNotBlank(query.getVirtualName())) {
                sql.append(" and v.name like '").append(query.getVirtualName()).append("%'");
            }
            sql.append(" )");
        } else {
            if (StringUtils.isNotBlank(query.getVirtualName())) {
                sql.append(" and exists(");
                sql.append(" select 1 from ec_oe_exam_capture_camera_info v where v.exam_record_data_id = rd.id");
                sql.append(" and v.name like '").append(query.getVirtualName()).append("%'");
                sql.append(" )");
            }
        }

        if (!isCount) {
            sql.append(" group by rd.id");

            if (query.getUpdateTimeDesc() != null && query.getUpdateTimeDesc()) {
                sql.append(" order by rd.update_time desc");
            } else {
                // 默认排序
                sql.append(" order by rd.id desc");
            }
        }

        return sql.toString();
    }

}
