package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordPaperStructRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordPaperStructEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordPaperStructService;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author chenken
 * @date 2018年11月13日 上午11:52:02
 * @company QMTH
 * @description 试卷结构服务类
 */
@Service("examRecordPaperStructService")
public class ExamRecordPaperStructServiceImpl implements ExamRecordPaperStructService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordPaperStructServiceImpl.class);

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamRecordPaperStructRepo examRecordPaperStructRepo;

    @Override
    public ExamRecordPaperStructEntity getExamRecordPaperStruct(Long examRecordDataId) {
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData == null) {
            throw new StatusException("400404", "未找到考试记录数据");
        }

        String paperStructId = examRecordData.getPaperStructId();
        if (StringUtils.isBlank(paperStructId)) {
            log.warn("未找到考试试卷结构！paperStructId is null, basePaperId:{} examType:{} examRecordDataId:{}",
                    examRecordData.getBasePaperId(), examRecordData.getExamType(), examRecordData.getId());
            throw new StatusException("400404", "未找到考试试卷结构");
        } else {
            return GlobalHelper.getEntity(examRecordPaperStructRepo, paperStructId, ExamRecordPaperStructEntity.class);
        }
    }

}
